/*-
 * Copyright (c) 2008 Joe Marcus Clarke.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $MCom: pbi-thumbnailer/pbi-thumbnail.c,v 1.3 2008/06/08 19:00:16 marcus Exp $
 *
 */

#include <gtk/gtk.h>
#include <glib.h>
#include <gio/gio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>

static char **files = NULL;

static const GOptionEntry entries[] = {
	 { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &files, NULL, "[FILE...]" },
	 { NULL }
};

int
main (int argc, char **argv)
{
	GOptionContext *context;
	GError *err = NULL;
	GdkPixbuf *pixbuf = NULL;
	GdkPixbufLoader *loader;
	GFile *file;
	GFileInputStream *istream;
	GDataInputStream *distream;
	int ret = 1;
	gboolean found_icon = FALSE;
	gboolean custom_icon = FALSE;
	char *input, *output;

	context = g_option_context_new ("Thumbnail PBIs");
	g_option_context_add_main_entries (context, entries, "pbi-thumbnailer");
	g_type_init ();

	if (g_option_context_parse (context, &argc, &argv, &err) == FALSE) {
		g_print ("Could not parse command line options: %s\n", err->message);
		g_error_free (err);
		return ret;
	}

	if (files == NULL || g_strv_length (files) != 2) {
		g_print ("Input and output filenames are missing\n");
		return ret;
	}

	input = files[0];
	output = files[1];

	file = g_file_new_for_uri (input);

	istream = g_file_read (file, NULL, &err);
	if (istream == NULL) {
		g_print ("Failed to open %s for reading: %s\n", input, err->message);
		g_error_free (err);
		g_object_unref (file);
		return ret;
	}

	distream = g_data_input_stream_new (G_INPUT_STREAM (istream));

	loader = gdk_pixbuf_loader_new_with_type ("png", NULL);

	while (TRUE) {
		char *line;
		guchar *buffer;
		gsize len;

		line = g_data_input_stream_read_until (distream, "\n", &len, NULL, &err);
		if (line == NULL) {
			g_print ("Error encountered reading from %s: %s\n", input, err->message);
			g_error_free (err);
			goto error;
		}

		buffer = g_new (guchar, len + 1);
		memcpy (buffer, line, len);
		buffer[len] = '\n';

		if (! found_icon) {
			if (! strcmp (line, "__PBI_ICON__")) {
				found_icon = TRUE;
			} else if (! strcmp (line, "__PBI_ARCHIVE__")) {
				g_free (line);
				g_free (buffer);
				break;
			}
			g_free (line);
			g_free (buffer);
			continue;
		}

		if (! strcmp (line, "__PBI_ARCHIVE__")) {
			if (! gdk_pixbuf_loader_close (loader, &err)) {
				g_print ("Error closing pixbuf loader: %s\n", err->message);
				g_error_free (err);
				g_free (buffer);
				g_free (line);
				goto error;
			}
			custom_icon = TRUE;
			g_free (line);
			g_free (buffer);
			break;
		}

		g_free (line);

		if (! gdk_pixbuf_loader_write (loader, (const guchar *) buffer,
					 len + 1, &err)) {
			g_print ("Failed to write to pixbuf loader: %s\n", err->message);
			g_free (buffer);
			g_error_free (err);
			goto error;
		}

		g_free (buffer);
	}

	if (custom_icon) {
		char *height, *width;

		pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
		if (pixbuf == NULL) {
			g_print ("Error retrieving thumbnail from pixbuf loader\n");
			goto error;
		}

		height = g_strdup_printf ("%d", gdk_pixbuf_get_height (pixbuf));
		width = g_strdup_printf ("%d", gdk_pixbuf_get_width (pixbuf));

		if (! gdk_pixbuf_save (pixbuf, output, "png", &err,
				       "tEXt::Thumb::Image::Width", width,
				       "tEXt::Thumb::Image::Height", height,
				       NULL)) {
			g_print ("Failed to save thumbnail to %s: %s\n", output, err->message);
			g_error_free (err);
			g_free (height);
			g_free (width);
			goto error;
		}

		g_free (height);
		g_free (width);

		ret = 0;
	}

error:
	g_input_stream_close (G_INPUT_STREAM (distream), NULL, NULL);
	g_input_stream_close (G_INPUT_STREAM (istream), NULL, NULL);
	g_object_unref (file);
	if (pixbuf != NULL)
		g_object_unref (pixbuf);

	return ret;
}
